@@ -1,15 +1,39 @@
 #' Parse R code
 #'
+#' @description
 #' These functions parse and transform text into R expressions. This
 #' is the first step to interpret or evaluate a piece of R code
 #' written by a programmer.
 #'
-#' `parse_expr()` returns one expression. If the text contains more
-#' than one expression (separated by semicolons or new lines), an error is
-#' issued. On the other hand `parse_exprs()` can handle multiple
-#' expressions. It always returns a list of expressions (compare to
-#' [base::parse()] which returns a base::expression vector). All
-#' functions also support R connections.
+#' * `parse_expr()` returns one expression. If the text contains more
+#'   than one expression (separated by semicolons or new lines), an
+#'   error is issued. On the other hand `parse_exprs()` can handle
+#'   multiple expressions. It always returns a list of expressions
+#'   (compare to [base::parse()] which returns a base::expression
+#'   vector). All functions also support R connections.
+#'
+#' * `parse_expr()` concatenates `x` with `\\n` separators prior to
+#'   parsing in order to support the roundtrip
+#'   `parse_expr(expr_deparse(x))` (deparsed expressions might be
+#'   multiline). On the other hand, `parse_exprs()` doesn't do any
+#'   concatenation because it's designed to support named inputs. The
+#'   names are matched to the expressions in the output, which is
+#'   useful when a single named string creates multiple expressions.
+#'
+#'   In other words, `parse_expr()` supports vector of lines whereas
+#'   `parse_exprs()` expects vectors of complete deparsed expressions.
+#'
+#' * `parse_quo()` and `parse_quos()` are variants that create a
+#'   [quosure][quo]. Supply `env = current_env()` if you're parsing
+#'   code to be evaluated in your current context. Supply `env =
+#'   global_env()` when you're parsing external user input to be
+#'   evaluated in user context.
+#'
+#'   Unlike quosures created with [enquo()], [enquos()], or `{{`, a
+#'   parsed quosure never contains injected quosures. It is thus safe
+#'   to evaluate them with `eval()` instead of [eval_tidy()], though
+#'   the latter is more convenient as you don't need to extract `expr`
+#'   and `env`.
 #'
 #' @param x Text containing expressions to parse_expr for
 #'   `parse_expr()` and `parse_exprs()`. Can also be an R connection,
@@ -21,6 +45,7 @@
 #'   length of the input. This would happen is one of the strings
 #'   contain several expressions (such as `"foo; bar"`). The names of
 #'   `x` are preserved (and recycled in case of multiple expressions).
+#'   The `_quo` suffixed variants return quosures.
 #' @seealso [base::parse()]
 #' @export
 #' @examples
@@ -41,13 +66,19 @@
 #' # We can now parse it by supplying a connection:
 #' parse_exprs(file(path))
 parse_expr <- function(x) {
-  exprs <- parse_exprs(x)
+  if (is_character(x)) {
+    exprs <- parse(text = paste_line(x))
+  } else {
+    exprs <- parse_exprs(x)
+  }
 
   n <- length(exprs)
-  if (n == 0) {
-    abort("No expression to parse")
-  } else if (n > 1) {
-    abort("More than one expression parsed")
+  if (n != 1) {
+    abort(sprintf(
+      "%s must contain exactly 1 expression, not %d.",
+      format_arg("x"),
+      n
+    ))
   }
 
   exprs[[1]]
@@ -64,7 +95,7 @@ parse_exprs <- function(x) {
   } else if (is.character(x)) {
     exprs <- chr_parse_exprs(x)
   } else {
-    abort("`x` must be a character vector or an R connection")
+    stop_input_type(x, "a character vector or an R connection")
   }
   as.list(exprs)
 }
@@ -76,41 +107,30 @@ chr_parse_exprs <- function(x) {
   parsed <- unname(parsed)
 
   if (!is_null(nms)) {
-    nms <- flatten_chr(map2(parsed, nms, rep_along))
+    nms <- list_c(map2(parsed, nms, rep_along))
+  }
+  if (length(parsed)) {
+    parsed <- list_c(parsed)
   }
-  parsed <- flatten(parsed)
 
   set_names(parsed, nms)
 }
 
-#' Parsing variants that return quosures
-#'
-#' @description
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
-#'
-#' These functions are in the questioning stage because they don't add
-#' value compared to using [parse_expr()] with [new_quosure()].
-#'
-#' @inheritParams parse_expr
-#' @param env The environment for the quosures. Depending on the use
-#'   case, a good default might be the [global
-#'   environment][global_env] but you might also want to evaluate the
-#'   R code in an isolated context (perhaps a child of the global
-#'   environment or of the [base environment][base_env]).
-#' @keywords internal
+#' @rdname parse_expr
+#' @param env The environment for the quosures. The [global
+#'   environment][global_env] (the default) may be the right choice
+#'   when you are parsing external user inputs. You might also want to
+#'   evaluate the R code in an isolated context (perhaps a child of
+#'   the global environment or of the [base environment][base_env]).
 #' @export
 parse_quo <- function(x, env) {
-  if (missing(env)) {
-    abort("The quosure environment should be explicitly supplied as `env`")
-  }
+  check_required(env)
   new_quosure(parse_expr(x), as_environment(env))
 }
-#' @rdname parse_quo
+#' @rdname parse_expr
 #' @export
 parse_quos <- function(x, env) {
-  if (missing(env)) {
-    abort("The quosure environment should be explicitly supplied as `env`")
-  }
+  check_required(env)
   out <- map(parse_exprs(x), new_quosure, env = as_environment(env))
   new_quosures(out)
 }
