@@ -287,6 +287,9 @@ normalise_parents <- function(parents) {
 winch_available_env <- new.env(parent = emptyenv())
 
 add_winch_trace <- function(trace) {
+  # FIXME: Until r-prof/winch#56 is fixed
+  return(trace)
+
   avail <- winch_available_env$installed
   if (is_null(avail)) {
     avail <- rlang::is_installed("winch")
@@ -432,11 +435,7 @@ format.rlang_trace <- function(x,
 
 arg_match_simplify <- function(simplify, call = caller_env()) {
   if (is_null(simplify)) {
-    if (use_tree_display()) {
-      return("none")
-    } else {
-      return("branch")
-    }
+    return("none")
   }
 
   if (is_string(simplify, "collapse")) {
@@ -446,15 +445,11 @@ arg_match_simplify <- function(simplify, call = caller_env()) {
   arg_match0(simplify, c("none", "branch"), error_call = call)
 }
 arg_match_drop <- function(drop) {
-  if (is_null(drop)) {
-    use_tree_display()
-  } else {
-    drop
-  }
+  drop %||% TRUE
 }
 
 deprecate_collapse <- function() {
-  warn_deprecated("`\"collapse\"` is deprecated as of rlang 1.1.0.\nPlease use `\"none\"` instead.")
+  deprecate_warn("`\"collapse\"` is deprecated as of rlang 1.1.0.\nPlease use `\"none\"` instead.")
 }
 
 trace_format <- function(trace,
@@ -508,7 +503,7 @@ cli_branch <- function(tree,
   indices <- paste0(" ", indices, ". ")
   padding <- spaces(nchar(indices[[1]]))
 
-  lines <- paste0(silver(indices), lines)
+  lines <- paste0(col_silver(indices), lines)
 
   src_locs <- tree$src_loc
   src_locs <- map_if(src_locs, nzchar, ~ paste0(padding, "  at ", .x))
@@ -558,7 +553,7 @@ style_locs <- function(locs) {
   chr(map_if(locs, nzchar, col_grey))
 }
 zip_chr <- function(xs, ys) {
-  flatten_chr(map2(xs, ys, function(x, y) {
+  list_c(map2(xs, ys, function(x, y) {
     if (nzchar(y)) {
       c(x, y)
     } else {
@@ -766,7 +761,7 @@ trace_as_tree <- function(trace,
     trace$node_type <- rep_len("main", nrow(trace))
   }
 
-  if (has_crayon()) {
+  if (has_ansi()) {
     # Detect runs of namespaces/global
     ns <- trace$namespace
     ns <- ifelse(is.na(ns) & trace$scope == "global", "global", ns)
@@ -868,7 +863,7 @@ src_loc <- function(srcref) {
 
   style_hyperlink(
     paste0(file_trim, ":", line, ":", column),
-    paste0("file://", file),
+    paste0("file://", normalizePath(file, mustWork = FALSE)),
     params = c(line = line, col = column)
   )
 }
